#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    from unittest import mock
except ImportError:
    import mock
import pytest

from tdclient import api
from tdclient.test.test_helper import *

def setup_function(function):
    unset_environ()

def test_import_with_id_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "elapsed_time": "3.14"
        }
    """
    td.put = mock.MagicMock(return_value=make_response(200, body))
    elapsed_time = td.import_data("db", "table", "format", b"stream", 6, unique_id="unique_id")
    td.put.assert_called_with("/v3/table/import_with_id/db/table/unique_id/format", b"stream", 6, endpoint="https://api-import.treasuredata.com/")
    assert elapsed_time == 3.14

def test_import_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "elapsed_time": 2.71
        }
    """
    td.put = mock.MagicMock(return_value=make_response(200, body))
    elapsed_time = td.import_data("db", "table", "format", b"stream", 6)
    td.put.assert_called_with("/v3/table/import/db/table/format", b"stream", 6, endpoint="https://api-import.treasuredata.com/")
    assert elapsed_time == 2.71

def test_import_failure():
    td = api.API("APIKEY")
    td.put = mock.MagicMock(return_value=make_response(500, b"error"))
    with pytest.raises(api.APIError) as error:
        td.import_data("db", "table", "format", b"stream", 6)
    assert error.value.args == ("500: Import failed: error",)



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    from urllib.parse import quote as urlquote # >=3.0
except ImportError:
    from urllib import quote as urlquote

class ExportAPI(object):
    ####
    ## Export API
    ##

    def export_data(self, db, table, storage_type, params={}):
        """
        TODO: add docstring
        => jobId:str
        """
        params["storage_type"] = storage_type
        with self.post("/v3/export/run/%s/%s" % (urlquote(str(db)), urlquote(str(table))), params) as res:
            code, body = res.status, res.read()
            if code != 200:
                self.raise_error("Export failed", res, body)
            js = self.checked_json(body, ["job_id"])
            return str(js["job_id"])



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

import json

from tdclient import api
from tdclient import model

class Client(object):
    """API Client for Treasure Data Service
    """

    def __init__(self, *args, **kwargs):
        self._api = api.API(*args, **kwargs)

    @property
    def api(self):
        """
        Returns: an instance of :class:`tdclient.api.API`
        """
        return self._api

    @property
    def apikey(self):
        """
        Returns: API key string.
        """
        return self._api.apikey

    def server_status(self):
        """
        Returns: a string represents current server status.
        """
        return self.api.server_status()

    def create_database(self, db_name, **kwargs):
        """
        Params:
            db_name (str): name of a database to create

        Returns: `True` if success
        """
        return self.api.create_database(db_name, **kwargs)

    def delete_database(self, db_name):
        """
        Params:
            db_name (str): name of database to delete

        Returns: `True` if success
        """
        return self.api.delete_database(db_name)

    def account(self):
        """
        Returns: :class:`tdclient.model.Acount`
        """
        account_id, plan, storage, guaranteed_cores, maximum_cores, created_at = self.api.show_account()
        return model.Account(self, account_id, plan, storage, guaranteed_cores, maximum_cores, created_at)

    def core_utilization(self, _from, to):
        """
        TODO: add docstring
        """
        _from, to, interval, history = self.api.account_core_utilization(_from, to)
        return (_from, to, interval, history)

    def databases(self):
        """
        Returns: a list of :class:`tdclient.model.Database`
        """
        m = self.api.list_databases()
        return [ model.Database(self, db_name, None, *args) for (db_name, args) in m.items() ]

    def database(self, db_name):
        """
        Params:
            db_name (str): name of a database

        Returns: :class:`tdclient.model.Database`
        """
        m = self.api.list_databases()
        if db_name in m:
            return model.Database(self, db_name, None, *m[db_name])
        else:
            raise api.NotFoundError("Database '%s' does not exist" % (db_name))

    def create_log_table(self, db_name, table_name):
        """
        Params:
            db_name (str): name of a database
            table_name (str): name of a table to create

        Returns: `True` if success
        """
        return self.api.create_log_table(db_name, table_name)

    def create_item_table(self, db_name, table_name, primary_key, primary_key_type):
        """
        Params:
            db_name (str): name of a database
            table_name (str): name of a table to create
            primary_key (str): name of primary key column
            primary_key_type (str): type of primary key column

        Returns: `True` if success
        """
        return self.api.create_item_table(db_name, table_name, primary_key, primary_key_type)

    def swap_table(self, db_name, table_name1, table_name2):
        """
        Params:
            db_name (str): name of a database
            table_name1 (str): original table name
            table_name2 (str): table name you want to rename to

        Returns: `True` if success
        """
        return self.api.swap_table(db_name, table_name1, table_name2)

    def update_schema(self, db_name, table_name, schema):
        """
        TODO: add docstring
        => True
        """
        return self.api.update_schema(db_name, table_name, json.dumps(schema))

    def update_expire(self, db_name, table_name, expire_days):
        """
        TODO: add docstring
        => True
        """
        return self.api.update_expire(db_name, table_name, expire_days)

    def delete_table(self, db_name, table_name):
        """
        Params:
            db_name (str): name of a database
            table_name (str): name of a table

        Returns: a string represents the type of deleted table
        """
        return self.api.delete_table(db_name, table_name)

    def tables(self, db_name):
        """
        Params:
            db_name (str): name of a database

        Returns: a list of :class:`tdclient.model.Table`
        """
        m = self.api.list_tables(db_name)
        return [ model.Table(self, db_name, table_name, *args) for (table_name, args) in m.items() ]

    def table(self, db_name, table_name):
        """
        Params:
            db_name (str): name of a database
            table_name (str): name of a table

        Returns: :class:`tdclient.model.Table`

        Raises:
            tdclient.api.NotFoundError: if the table doesn't exist
        """
        tables = self.tables(db_name)
        for table in tables:
            if table.table_name == table_name:
                return table
        raise api.NotFoundError("Table '%s.%s' does not exist" % (db_name, table_name))

    def tail(self, db_name, table_name, count, to=None, _from=None, block=None):
        """
        TODO: add docstring
        """
        return self.api.tail(db_name, table_name, count, to, _from, block)

    def query(self, db_name, q, result_url=None, priority=None, retry_limit=None, type="hive", **kwargs):
        """
        Params:
            db_name (str): name of a database
            q (str): a query string
            result_url (str): result output URL
            priority (str): priority
            retry_limit (int): retry limit
            type (str): name of a query engine

        Returns: :class:`tdclient.model.Job`

        Raises:
            ValueError: if unknown query type has been specified
        """
        # for compatibility, assume type is hive unless specifically specified
        if type not in ["hive", "pig", "impala", "presto"]:
            raise ValueError("The specified query type is not supported: %s" % (type))
        job_id = self.api.query(q, type, db_name, result_url, priority, retry_limit, **kwargs)
        return model.Job(self, job_id, type, q)

    def jobs(self, _from=None, to=None, status=None, conditions=None):
        """
        Params:
            _from (int):
            to (int):
            status (str):
            conditions (str)

        Returns: a list of :class:`tdclient.model.Job`
        """
        results = self.api.list_jobs(_from, to, status, conditions)
        def job(lis):
            job_id, type, status, query, start_at, end_at, cpu_time, result_size, result_url, priority, retry_limit, org, db = lis
            return model.Job(self, job_id, type, query, status, None, None, start_at, end_at, cpu_time,
                             result_size, None, result_url, None, priority, retry_limit, org, db)
        return [ job(result) for result in results ]

    def job(self, job_id):
        """
        Params:
            job_id (int): job id

        Returns: :class:`tdclient.model.Job`
        """
        job_id = str(job_id)
        type, query, status, url, debug, start_at, end_at, cpu_time, result_size, result_url, hive_result_schema, priority, retry_limit, org, db = self.api.show_job(job_id)
        return model.Job(self, job_id, type, query, status, url, debug, start_at, end_at, cpu_time,
                         result_size, None, result_url, hive_result_schema, priority, retry_limit, org, db)

    def job_status(self, job_id):
        """
        TODO: add docstring
        => status:str
        """
        return self.api.job_status(job_id)

    def job_result(self, job_id):
        """
        TODO: add docstring
        => result:[{column:str=>value:object]
        """
        return self.api.job_result(job_id)

    def job_result_each(self, job_id):
        """
        TODO: add docstring
        => None
        """
        for row in self.api.job_result_each(job_id):
            yield row

    def job_result_format(self, job_id, format):
        """
        TODO: add docstring
        """
        return self.api.job_result_format(job_id, format)

    def job_result_format_each(self, job_id, format):
        """
        TODO: add docstring
        """
        for row in self.api.job_result_format_each(job_id, format):
            yield row

    def kill(self, job_id):
        """
        TODO: add docstring
        => former_status:str
        """
        return self.api.kill(job_id)

    def export_data(self, db_name, table_name, storage_type, opts={}):
        """
        TODO: add docstring
        => :class:`tdclient.model.Job`
        """
        job_id = self.api.export_data(db_name, table_name, storage_type, opts)
        return model.Job(self, job_id, "export", None)

    def partial_delete(self, db_name, table_name, to, _from, opts={}):
        """
        TODO: add docstring
        => :class:`tdclient.model.Job`
        """
        job_id = self.api.partial_delete(db_name, table_name, to, _from, opts)
        return model.Job(self, job_id, "partialdelete", None)

    def create_bulk_import(self, name, database, table, opts={}):
        """
        TODO: add docstring
        => None
        """
        return self.api.create_bulk_import(name, database, table, opts)

    def delete_bulk_import(self, name):
        """
        TODO: add docstring
        => None
        """
        return self.api.delete_bulk_import(name)

    def freeze_bulk_import(self, name):
        """
        TODO: add docstring
        => None
        """
        return self.api.freeze_bulk_import(name)

    def unfreeze_bulk_import(self, name):
        """
        TODO: add docstring
        => None
        """
        return self.api.unfreeze_bulk_import(name)

    def perform_bulk_import(self, name):
        """
        TODO: add docstring
        => :class:`tdclient.model.Job`
        """
        job_id = self.api.perform_bulk_import(name)
        return model.Job(self, job_id, "bulk_import", None)

    def commit_bulk_import(self, name):
        """
        TODO: add docstring
        => None
        """
        return self.api.commit_bulk_import(name)

    def bulk_import_error_records(self, name):
        """
        TODO: add docstring
        => records:[row:dict]
        """
        for record in self.api.bulk_import_error_records(name):
            yield record

    def bulk_import(self, name):
        """
        TODO: add docstring
        => :class:`tdclient.model.BulkImport`
        """
        data = self.api.show_bulk_import(name)
        return model.BulkImport(self, **data)

    def bulk_imports(self):
        """
        TODO: add docstring
        => [:class:`tdclient.model.BulkImport`]
        """
        return [ model.BulkImport(self, **data) for data in self.api.list_bulk_imports() ]

    def bulk_import_upload_part(self, name, part_name, stream, size):
        """
        TODO: add docstring
        => None
        """
        return self.api.bulk_import_upload_part(name, part_name, stream, size)

    def bulk_import_delete_part(self, name, part_name):
        """
        TODO: add docstring
        => None
        """
        return self.api.bulk_import_delete_part(name, part_name)

    def list_bulk_import_parts(self, name):
        """
        TODO: add docstring
        """
        return self.api.list_bulk_import_parts(name)

    def create_schedule(self, name, params={}):
        """
        TODO: add docstring
        => first_time:datetime.datetime
        """
        if "cron" not in params:
            raise ValueError("'cron' option is required")
        if "query" not in params:
            raise ValueError("'query' option is required")
        start = self.api.create_schedule(name, params)
        return start # TODO: parse datetime string

    def delete_schedule(self, name):
        """
        TODO: add docstring
        => True
        """
        return self.api.delete_schedule(name)

    def schedules(self):
        """
        TODO: add docstring
        [:class:`tdclient.model.Schedule`]
        """
        result = self.api.list_schedules()
        def schedule(m):
            name,cron,query,database,result_url,timezone,delay,next_time,priority,retry_limit,org_name = m
            return model.Schedule(self, name, cron, query, database, result_url, timezone, delay, next_time, priority, retry_limit, org_name)
        return [ schedule(m) for m in result ]

    def update_schedule(self, name, params={}):
        """
        TODO: add docstring
        [:class:`tdclient.model.ScheduledJob`]
        """
        self.api.update_schedule(name, params)

    def history(self, name, _from=None, to=None):
        """
        TODO: add docstring
        [:class:`tdclient.model.ScheduledJob`]
        """
        result = self.api.history(name, _from, to)
        def scheduled_job(m):
            scheduled_at,job_id,type,status,query,start_at,end_at,result_url,priority,database = m
            job_param = [job_id, type, query, status,
                None, None, # url, debug
                start_at, end_at,
                None, # cpu_time
                None, None, # result_size, result
                result_url,
                None, # hive_result_schema
                priority,
                None, # retry_limit
                None, # TODO org_name
                database]
            return model.ScheduledJob(self, scheduled_at, *job_param)
        return [ scheduled_job(m) for m in result ]

    def run_schedule(self, name, time, num):
        """
        TODO: add docstring
        [:class:`tdclient.model.ScheduledJob`]
        """
        results = self.api.run_schedule(name, time, num)
        def scheduled_job(m):
            job_id,type,scheduled_at = m
            return model.ScheduledJob(self, scheduled_at, job_id, type, None)
        return [ scheduled_job(m) for m in results ]

    def import_data(self, db_name, table_name, format, stream, size, unique_id=None):
        """
        TODO: add docstring
        => time:flaot
        """
        return self.api.import_data(db_name, table_name, format, stream, size, unique_id)

    def results(self):
        """
        TODO: add docstring
        => [:class:`tdclient.model.Result`]
        """
        results = self.api.list_result()
        def result(m):
            name,url,organizations = m
            return model.Result(self, name, url, organizations)
        return [ result(m) for m in results ]

    def create_result(self, name, url, params={}):
        """
        TODO: add docstring
        => True
        """
        return self.api.create_result(name, url, params)

    def delete_result(self, name):
        """
        TODO: add docstring
        => True
        """
        return self.api.delete_result(name)

    def users(self):
        """
        TODO: add docstring
        => [:class:`tdclient.model.User`]
        """
        results = self.api.list_users()
        def user(m):
            name,org,roles,email = m
            return model.User(self, name, org, roles, email)
        return [ user(m) for m in results ]

    def add_user(self, name, org, email, password):
        """
        TODO: add docstring
        => True
        """
        return self.api.add_user(name, org, email, password)

    def remove_user(self, user):
        """
        TODO: add docstring
        => True
        """
        return self.api.remove_user(user)

    def change_email(self, user, email):
        """
        TODO: add docstring
        => True
        """
        return self.api.change_email(user, email)

    def list_apikeys(self, user):
        """
        TODO: add docstring
        => [apikey:str]
        """
        return self.api.list_apikeys(user)

    def add_apikey(self, user):
        """
        TODO: add docstring
        => True
        """
        return self.api.add_apikey(user)

    def remove_apikey(self, user, apikey):
        """
        TODO: add docstring
        => True
        """
        return self.api.remove_apikey(user, apikey)

    def change_password(self, user, password):
        """
        TODO: add docstring
        => True
        """
        return self.api.change_password(user, password)

    def change_my_password(self, old_password, password):
        """
        TODO: add docstring
        => True
        """
        return self.api.change_my_password(old_password, password)

    def access_controls(self):
        """
        TODO: add docstring
        => [:class:`tdclient.model.User`]
        """
        results = self.api.list_access_controls()
        def access_control(m):
            subject,action,scope,grant_option = m
            return model.AccessControl(self, subject, action, scope, grant_option)
        return [ access_control(m) for m in results ]

    def grant_access_control(self, subject, action, scope, grant_option):
        """
        TODO: add docstring
        => True
        """
        return self.api.grant_access_control(subject, action, scope, grant_option)

    def revoke_access_control(self, subject, action, scope):
        """
        TODO: add docstring
        => True
        """
        return self.api.revoke_access_control(subject, action, scope)

    def test_access_control(self, user, action, scope):
        """
        TODO: add docstring
        => True
        """
        return self.api.test_access_control(user, action, scope)

    def close(self):
        """
        TODO: add docstring
        """
        return self._api.close()



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    from unittest import mock
except ImportError:
    import mock
import pytest

from tdclient import api
from tdclient.test.test_helper import *

def setup_function(function):
    unset_environ()

def test_create_schedule_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "start": "foo"
        }
    """
    td.post = mock.MagicMock(return_value=make_response(200, body))
    start = td.create_schedule("bar", {"type": "presto"})
    td.post.assert_called_with("/v3/schedule/create/bar", {"type": "presto"})
    assert start == "foo"

def test_delete_schedule_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "cron": "foo",
            "query": "SELECT 1 FROM nasdaq"
        }
    """
    td.post = mock.MagicMock(return_value=make_response(200, body))
    cron, query = td.delete_schedule("bar")
    td.post.assert_called_with("/v3/schedule/delete/bar")
    assert cron == "foo"
    assert query == "SELECT 1 FROM nasdaq"

def test_list_schedules_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "schedules":[
                {"name":"foo","cron":"* * * * *","query":"SELECT COUNT(1) FROM nasdaq;","database":"sample_datasets","result":"","timezone":"UTC","delay":"","next_time":"","priority":"","retry_limit":""},
                {"name":"bar","cron":"* * * * *","query":"SELECT COUNT(1) FROM nasdaq;","database":"sample_datasets","result":"","timezone":"UTC","delay":"","next_time":"","priority":"","retry_limit":""},
                {"name":"baz","cron":"* * * * *","query":"SELECT COUNT(1) FROM nasdaq;","database":"sample_datasets","result":"","timezone":"UTC","delay":"","next_time":"","priority":"","retry_limit":""}
            ]
        }
    """
    td.get = mock.MagicMock(return_value=make_response(200, body))
    schedules = td.list_schedules()
    td.get.assert_called_with("/v3/schedule/list")
    assert len(schedules) == 3

def test_list_schedules_failure():
    td = api.API("APIKEY")
    td.get = mock.MagicMock(return_value=make_response(500, b"error"))
    with pytest.raises(api.APIError) as error:
        td.list_schedules()
    assert error.value.args == ("500: List schedules failed: error",)

def test_update_schedule_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    td.update_schedule("foo")
    td.post.assert_called_with("/v3/schedule/update/foo", {})

def test_history_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "history": [
                {"job_id":"12345"},
                {"job_id":"67890"}
            ]
        }
    """
    td.get = mock.MagicMock(return_value=make_response(200, body))
    history = td.history("foo", 0, 3)
    td.get.assert_called_with("/v3/schedule/history/foo", {"from": "0", "to": "3"})

def test_run_schedule_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "jobs": [
                {"job_id":"12345","type":"hive"},
                {"job_id":"67890","type":"hive"}
            ]
        }
    """
    td.post = mock.MagicMock(return_value=make_response(200, body))
    jobs = td.run_schedule("name", "time", 1)
    td.post.assert_called_with("/v3/schedule/run/name/time", {"num": 1})



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    from unittest import mock
except ImportError:
    import mock
import os
import pytest
try:
    import urllib.parse as urlparse
except ImportError:
    import urlparse
import urllib3

from tdclient import api
from tdclient import version
from tdclient.test.test_helper import *

def setup_function(function):
    unset_environ()

def test_apikey_success():
    td = api.API("apikey")
    assert td.apikey == "apikey"

def test_apikey_from_environ():
    os.environ["TD_API_KEY"] = "APIKEY"
    td = api.API()
    assert td.apikey == "APIKEY"

def test_apikey_failure():
    with pytest.raises(ValueError) as error:
        api.API()
    assert error.value.args == ("no API key given",)

def test_default_user_agent():
    td = api.API("apikey")
    assert td._user_agent.startswith("TD-Client-Python: %s" % version.__version__)

def test_user_agent_from_keyword():
    td = api.API("apikey", user_agent="user_agent")
    assert td._user_agent == "user_agent"

def test_default_endpoint():
    td = api.API("apikey")
    assert isinstance(td.http, urllib3.PoolManager)
    url, headers = td.build_request()
    assert "https://api.treasuredata.com/" == url

def test_endpoint_from_environ():
    os.environ["TD_API_SERVER"] = "http://api1.example.com"
    td = api.API("apikey")
    assert isinstance(td.http, urllib3.PoolManager)
    url, headers = td.build_request()
    assert "http://api1.example.com" == url

def test_endpoint_from_keyword():
    td = api.API("apikey", endpoint="http://api2.example.com")
    assert isinstance(td.http, urllib3.PoolManager)
    url, headers = td.build_request()
    assert "http://api2.example.com" == url

def test_endpoint_prefer_keyword():
    os.environ["TD_API_SERVER"] = "http://api1.example.com"
    td = api.API("apikey", endpoint="http://api2.example.com")
    assert isinstance(td.http, urllib3.PoolManager)
    url, headers = td.build_request()
    assert "http://api2.example.com" == url

def test_http_endpoint_with_custom_port():
    td = api.API("apikey", endpoint="http://api.example.com:8080/")
    assert isinstance(td.http, urllib3.PoolManager)
    url, headers = td.build_request()
    assert "http://api.example.com:8080/" == url

def test_https_endpoint():
    td = api.API("apikey", endpoint="https://api.example.com/")
    assert isinstance(td.http, urllib3.PoolManager)
    url, headers = td.build_request()
    assert "https://api.example.com/" == url

def test_https_endpoint_with_custom_path():
    td = api.API("apikey", endpoint="https://api.example.com/v1/")
    assert isinstance(td.http, urllib3.PoolManager)
    url, headers = td.build_request()
    assert "https://api.example.com/v1/" == url

def test_http_proxy_from_environ():
    os.environ["HTTP_PROXY"] = "proxy1.example.com:8080"
    td = api.API("apikey")
    assert isinstance(td.http, urllib3.ProxyManager)
    assert td.http.proxy.url == "http://proxy1.example.com:8080"

def test_http_proxy_from_keyword():
    td = api.API("apikey", http_proxy="proxy2.example.com:8080")
    assert isinstance(td.http, urllib3.ProxyManager)
    assert td.http.proxy.url == "http://proxy2.example.com:8080"

def test_http_proxy_prefer_keyword():
    os.environ["HTTP_PROXY"] = "proxy1.example.com:8080"
    td = api.API("apikey", http_proxy="proxy2.example.com:8080")
    assert isinstance(td.http, urllib3.ProxyManager)
    assert td.http.proxy.url == "http://proxy2.example.com:8080"

def test_http_proxy_with_scheme():
    os.environ["HTTP_PROXY"] = "http://proxy1.example.com:8080/"
    td = api.API("apikey")
    assert isinstance(td.http, urllib3.ProxyManager)
    assert td.http.proxy.url == "http://proxy1.example.com:8080/"

def test_timeout():
    with mock.patch("tdclient.api.urllib3") as urllib3:
        td = api.API("apikey", timeout=12345)
        assert urllib3.PoolManager.called
        args, kwargs = urllib3.PoolManager.call_args
        assert kwargs["timeout"] == 12345

def test_connect_timeout():
    with mock.patch("tdclient.api.urllib3") as urllib3:
        td = api.API("apikey", connect_timeout=1, read_timeout=2, send_timeout=4)
        assert urllib3.PoolManager.called
        args, kwargs = urllib3.PoolManager.call_args
        assert kwargs["timeout"] == 7

def test_get_success():
    td = api.API("APIKEY")
    td.sleep = mock.MagicMock()
    td.http.request = mock.MagicMock()
    responses = [
        make_raw_response(200, b"body"),
    ]
    td.http.request.side_effect = responses
    with td.get("/foo", {"bar": "baz"}) as response:
        args, kwargs = td.http.request.call_args
        assert args == ("GET", "https://api.treasuredata.com/foo")
        assert kwargs["fields"] == {"bar": "baz"}
        assert sorted(kwargs["headers"].keys()) == ["accept-encoding", "authorization", "date", "user-agent"]
        status, body = response.status, response.read()
        assert status == 200
        assert body == b"body"
    assert not td.sleep.called

def test_get_error():
    td = api.API("APIKEY")
    td.sleep = mock.MagicMock()
    td.http.request = mock.MagicMock()
    responses = [
        make_raw_response(404, b"not found"),
    ]
    td.http.request.side_effect = responses
    with td.get("/foo", {"bar": "baz"}) as response:
        args, kwargs = td.http.request.call_args
        assert args == ("GET", "https://api.treasuredata.com/foo")
        assert kwargs["fields"] == {"bar": "baz"}
        assert sorted(kwargs["headers"].keys()) == ["accept-encoding", "authorization", "date", "user-agent"]
        status, body = response.status, response.read()
        assert status == 404
        assert body == b"not found"
    assert not td.sleep.called

def test_get_retry_success():
    td = api.API("APIKEY")
    td.sleep = mock.MagicMock()
    td.http.request = mock.MagicMock()
    responses = [
        make_raw_response(500, b"failure1"),
        make_raw_response(503, b"failure2"),
        make_raw_response(200, b"success1"),
    ]
    td.http.request.side_effect = responses
    with td.get("/foo", {"bar": "baz"}) as response:
        args, kwargs = td.http.request.call_args
        assert args == ("GET", "https://api.treasuredata.com/foo")
        assert kwargs["fields"] == {"bar": "baz"}
        assert sorted(kwargs["headers"].keys()) == ["accept-encoding", "authorization", "date", "user-agent"]
        status, body = response.status, response.read()
        assert status == 200
        assert body == b"success1"
        assert td.sleep.called
        sleeps = [ args[0] for (args, kwargs) in td.sleep.call_args_list ]
        assert len(sleeps) == len(responses) - 1
        assert sum(sleeps) < td._max_cumul_retry_delay

def test_get_failure():
    td = api.API("APIKEY")
    td.sleep = mock.MagicMock()
    td.http.request = mock.MagicMock()
    td.http.request.return_value = make_raw_response(500, b"failure")
    with pytest.raises(api.APIError) as error:
        with td.get("/foo", {"bar": "baz"}) as response:
            pass
    assert td.sleep.called
    sleeps = [ args[0] for (args, kwargs) in td.sleep.call_args_list ]
    assert td._max_cumul_retry_delay < sum(sleeps)

def test_post_success():
    td = api.API("APIKEY")
    td.sleep = mock.MagicMock()
    td.http.request = mock.MagicMock()
    responses = [
        make_raw_response(200, b"body"),
    ]
    td.http.request.side_effect = responses
    with td.post("/foo", {"bar": "baz"}) as response:
        args, kwargs = td.http.request.call_args
        assert args == ("POST", "https://api.treasuredata.com/foo")
        assert kwargs["fields"] == {"bar": "baz"}
        assert sorted(kwargs["headers"].keys()) == ["authorization", "date", "user-agent"]
        status, body = response.status, response.read()
        assert status == 200
        assert body == b"body"
    assert not td.sleep.called

def test_post_retry_success():
    td = api.API("APIKEY", retry_post_requests=True)
    td.sleep = mock.MagicMock()
    td.http.request = mock.MagicMock()
    responses = [
        make_raw_response(500, b"failure1"),
        make_raw_response(503, b"failure2"),
        make_raw_response(200, b"success1"),
    ]
    td.http.request.side_effect = responses
    with td.post("/foo", {"bar": "baz"}) as response:
        args, kwargs = td.http.request.call_args
        assert args == ("POST", "https://api.treasuredata.com/foo")
        assert kwargs["fields"] == {"bar": "baz"}
        assert sorted(kwargs["headers"].keys()) == ["authorization", "date", "user-agent"]
        status, body = response.status, response.read()
        assert status == 200
        assert body == b"success1"
        assert td.sleep.called
        sleeps = [ args[0] for (args, kwargs) in td.sleep.call_args_list ]
        assert len(sleeps) == len(responses) - 1
        assert sum(sleeps) < td._max_cumul_retry_delay

def test_post_never_retry():
    td = api.API("APIKEY", retry_post_requests=False)
    td.sleep = mock.MagicMock()
    td.http.request = mock.MagicMock()
    responses = [
        make_raw_response(500, b"failure"),
    ]
    td.http.request.side_effect = responses
    with pytest.raises(api.APIError) as error:
        with td.post("/foo", {"bar": "baz"}) as response:
            pass
    assert not td.sleep.called

def test_post_failure():
    td = api.API("APIKEY", retry_post_requests=True)
    td.sleep = mock.MagicMock()
    td.http.request = mock.MagicMock()
    td.http.request.return_value = make_raw_response(500, b"failure")
    with pytest.raises(api.APIError) as error:
        with td.post("/foo", {"bar": "baz"}) as response:
            pass
    assert td.sleep.called
    sleeps = [ args[0] for (args, kwargs) in td.sleep.call_args_list ]
    assert td._max_cumul_retry_delay < sum(sleeps)

def test_put_success():
    td = api.API("APIKEY")
    td.sleep = mock.MagicMock()
    td.http.urlopen = mock.MagicMock()
    responses = [
        make_raw_response(200, b"body"),
    ]
    td.http.urlopen.side_effect = responses
    with td.put("/foo", b"body", 7) as response:
        args, kwargs = td.http.urlopen.call_args
        assert args == ("PUT", "https://api.treasuredata.com/foo")
        assert sorted(kwargs["headers"].keys()) == ["authorization", "content-length", "content-type", "date", "user-agent"]
        status, body = response.status, response.read()
        assert status == 200
        assert body == b"body"
    assert not td.sleep.called

def test_put_retry_success():
    td = api.API("APIKEY")
    td.sleep = mock.MagicMock()
    td.http.urlopen = mock.MagicMock()
    responses = [
        make_raw_response(500, b"failure1"),
        make_raw_response(503, b"failure2"),
        make_raw_response(200, b"success1"),
    ]
    td.http.urlopen.side_effect = responses
    with td.put("/foo", b"body", 7) as response:
        args, kwargs = td.http.urlopen.call_args
        assert args == ("PUT", "https://api.treasuredata.com/foo")
        assert sorted(kwargs["headers"].keys()) == ["authorization", "content-length", "content-type", "date", "user-agent"]
        status, body = response.status, response.read()
        assert status == 200
        assert body == b"success1"
        assert td.sleep.called
        sleeps = [ args[0] for (args, kwargs) in td.sleep.call_args_list ]
        assert len(sleeps) == len(responses) - 1
        assert sum(sleeps) < td._max_cumul_retry_delay

def test_put_failure():
    td = api.API("APIKEY")
    td.sleep = mock.MagicMock()
    td.http.urlopen = mock.MagicMock()
    td.http.urlopen.return_value = make_raw_response(500, b"error")
    with pytest.raises(api.APIError) as error:
        with td.put("/foo", b"body", 7) as response:
            pass
    assert td.sleep.called
    sleeps = [ args[0] for (args, kwargs) in td.sleep.call_args_list ]
    assert td._max_cumul_retry_delay < sum(sleeps)

def test_raise_error_401():
    td = api.API("APIKEY")
    with pytest.raises(api.AuthError) as error:
        td.raise_error("msg", make_raw_response(401, b"unauthorized"), b"body")

def test_raise_error_403():
    td = api.API("APIKEY")
    with pytest.raises(api.ForbiddenError) as error:
        td.raise_error("msg", make_raw_response(403, b"forbidden"), b"body")

def test_raise_error_404():
    td = api.API("APIKEY")
    with pytest.raises(api.NotFoundError) as error:
        td.raise_error("msg", make_raw_response(404, b"not found"), b"body")

def test_raise_error_409():
    td = api.API("APIKEY")
    with pytest.raises(api.AlreadyExistsError) as error:
        td.raise_error("msg", make_raw_response(409, b"conflict"), b"body")

def test_raise_error_4xx():
    td = api.API("APIKEY")
    with pytest.raises(api.APIError) as error:
        td.raise_error("msg", make_raw_response(402, b"payment required"), b"body")

def test_checked_json_success():
    td = api.API("APIKEY")
    assert td.checked_json(b'{"foo": "FOO"}', ["foo"])
    assert td.checked_json(b'{"foo": "FOO", "bar": "BAR", "baz": "BAZ"}', ["foo"])
    assert td.checked_json(b'{"foo": "FOO", "bar": "BAR", "baz": "BAZ"}', ["foo", "bar"])

def test_checked_json_load_error():
    td = api.API("APIKEY")
    with pytest.raises(api.APIError) as error:
        td.checked_json(b'{invalid}', [])

def test_checked_json_field_error():
    td = api.API("APIKEY")
    with pytest.raises(api.APIError) as error:
        td.checked_json(b'{}', ["foo"])

def test_sleep():
    with mock.patch("tdclient.api.time") as time:
        td = api.API("apikey")
        td.sleep(600)
        assert time.sleep.called
        args, kwargs = time.sleep.call_args
        assert args == (600,)

def test_parsedate1():
    td = api.API("APIKEY")
    dt = td.parsedate("2013-11-01 16:48:41 -0700") # ???
    assert dt.year == 2013
    assert dt.month == 11
    assert dt.day == 1
    assert dt.hour == 16
    assert dt.minute == 48
    assert dt.second == 41
    assert dt.utcoffset().total_seconds() == -7.0 * 3600

def test_parsedate2():
    td = api.API("APIKEY")
    dt = td.parsedate("2013-11-13 19:39:19 UTC") # ???
    assert dt.year == 2013
    assert dt.month == 11
    assert dt.day == 13
    assert dt.hour == 19
    assert dt.minute == 39
    assert dt.second == 19
    assert dt.utcoffset().total_seconds() == 0.0 * 3600

def test_parsedate3():
    td = api.API("APIKEY")
    dt = td.parsedate("Sun Jun 26 17:39:18 -0400 2011") # rfc2882
    assert dt.year == 2011
    assert dt.month == 6
    assert dt.day == 26
    assert dt.hour == 17
    assert dt.minute == 39
    assert dt.second == 18
    assert dt.utcoffset().total_seconds() == -4.0 * 3600

def test_get_or_else():
    td = api.API("APIKEY")
    hashmap = {"foo": "FOO"}
    assert td.get_or_else(hashmap, "foo") == "FOO"
    assert td.get_or_else(hashmap, "foo", "abracadabra") == "FOO"
    assert td.get_or_else(hashmap, "bar") == None
    assert td.get_or_else(hashmap, "bar", "BAR") == "BAR"



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    from unittest import mock
except ImportError:
    import mock
import pytest

from tdclient import api
from tdclient.test.test_helper import *

def setup_function(function):
    unset_environ()

def test_list_result_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "results":[
              {"name":"foo","url":"http://example.com/1"},
              {"name":"bar","url":"http://example.com/2"},
              {"name":"baz","url":"http://example.com/3"}
            ]
        }
    """
    td.get = mock.MagicMock(return_value=make_response(200, body))
    results = td.list_result()
    td.get.assert_called_with("/v3/result/list")
    assert len(results) == 3

def test_list_result_success():
    td = api.API("APIKEY")
    td.get = mock.MagicMock(return_value=make_response(500, b"error"))
    with pytest.raises(api.APIError) as error:
        td.list_result()
    assert error.value.args == ("500: List result table failed: error",)

def test_create_result_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    results = td.create_result("foo", "http://example.com")
    td.post.assert_called_with("/v3/result/create/foo", {"url": "http://example.com"})

def test_delete_result_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    results = td.delete_result("foo")
    td.post.assert_called_with("/v3/result/delete/foo")



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    from unittest import mock
except ImportError:
    import mock
import pytest

from tdclient import api
from tdclient.test.test_helper import *

def setup_function(function):
    unset_environ()

def test_create_bulk_import_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    td.create_bulk_import("name", "db", "table")
    td.post.assert_called_with("/v3/bulk_import/create/name/db/table", {})

def test_delete_bulk_import_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    td.delete_bulk_import("name")
    td.post.assert_called_with("/v3/bulk_import/delete/name", {})

def test_show_bulk_import_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "status": "SUCCESS"
        }
    """
    td.get = mock.MagicMock(return_value=make_response(200, body))
    bulk_import = td.show_bulk_import("name")
    td.get.assert_called_with("/v3/bulk_import/show/name")
    assert bulk_import == {"status": "SUCCESS"}

def test_list_bulk_imports_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "bulk_imports":[
            ]
        }
    """
    td.get = mock.MagicMock(return_value=make_response(200, body))
    bulk_imports = td.list_bulk_imports()
    td.get.assert_called_with("/v3/bulk_import/list", {})
    assert len(bulk_imports) == 0

def test_list_bulk_imports_failure():
    td = api.API("APIKEY")
    td.get = mock.MagicMock(return_value=make_response(500, b"error"))
    with pytest.raises(api.APIError) as error:
        td.list_bulk_imports()
    assert error.value.args == ("500: List bulk imports failed: error",)

def test_list_bulk_import_parts_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "parts":[
            ]
        }
    """
    td.get = mock.MagicMock(return_value=make_response(200, body))
    parts = td.list_bulk_import_parts("foo")
    td.get.assert_called_with("/v3/bulk_import/list_parts/foo", {})
    assert len(parts) == 0

def test_list_bulk_import_upload_part_success():
    td = api.API("APIKEY")
    td.put = mock.MagicMock(return_value=make_response(200, b""))
    td.bulk_import_upload_part("name", "part_name", "stream", 1024)
    td.put.assert_called_with("/v3/bulk_import/upload_part/name/part_name", "stream", 1024)

def test_list_bulk_import_delete_part_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    td.bulk_import_delete_part("name", "part_name")
    td.post.assert_called_with("/v3/bulk_import/delete_part/name/part_name", {})

def test_freeze_bulk_import_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    td.freeze_bulk_import("name")
    td.post.assert_called_with("/v3/bulk_import/freeze/name", {})

def test_unfreeze_bulk_import_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    td.unfreeze_bulk_import("name")
    td.post.assert_called_with("/v3/bulk_import/unfreeze/name", {})

def test_perform_bulk_import_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "job_id": "12345"
        }
    """
    td.post = mock.MagicMock(return_value=make_response(200, body))
    job_id = td.perform_bulk_import("name")
    td.post.assert_called_with("/v3/bulk_import/perform/name", {})
    assert job_id == "12345"

def test_commit_bulk_import_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    td.commit_bulk_import("name")
    td.post.assert_called_with("/v3/bulk_import/commit/name", {})

#def test_bulk_import_error_records_success():
#    td = api.API("APIKEY")
#    # TODO: should be replaced by wire dump
#    td.post = mock.MagicMock(return_value=make_response(200, b""))
#    td.bulk_import_error_records("name")
#    td.post.assert_called_with("/v3/bulk_import/error_records/name", {})



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    from unittest import mock
except ImportError:
    import mock
import pytest

from tdclient import model
from tdclient.test.test_helper import *

def setup_function(function):
    unset_environ()

def test_account():
    client = mock.MagicMock()
    account = model.Account(client, 1, 2, storage_size=3456, guaranteed_cores=7, maximum_cores=8, created_at="created_at")
    assert account.account_id == 1
    assert account.plan == 2
    assert account.storage_size == 3456
    assert account.guaranteed_cores == 7
    assert account.maximum_cores == 8
    assert account.created_at == "created_at"

def test_account_storage_size_string():
    client = mock.MagicMock()
    account1 = model.Account(client, 1, 1, storage_size=0)
    assert account1.storage_size_string == "0.0 GB"
    account2 = model.Account(client, 1, 1, storage_size=50*1024*1024)
    assert account2.storage_size_string == "0.01 GB"
    account3 = model.Account(client, 1, 1, storage_size=50*1024*1024*1024)
    assert account3.storage_size_string == "50.0 GB"
    account4 = model.Account(client, 1, 1, storage_size=300*1024*1024*1024)
    assert account4.storage_size_string == "300 GB"

def test_database():
    client = mock.MagicMock()
    database = model.Database(client, "sample_datasets", tables=["nasdaq", "www_access"], count=12345, created_at="created_at", updated_at="updated_at", org_name="org_name", permission="administrator")
    assert database.org_name == "org_name"
    assert database.permission == "administrator"
    assert database.count == 12345
    assert database.name == "sample_datasets"
    assert database.tables() == ["nasdaq", "www_access"]
    assert database.created_at == "created_at"
    assert database.updated_at == "updated_at"

def test_table():
    client = mock.MagicMock()
    table = model.Table(client, "db_name", "table_name", "type", "schema", 12345, created_at="created_at", updated_at="updated_at", estimated_storage_size=67890, last_import="last_import", last_log_timestamp="last_log_timestamp", expire_days="expire_days", primary_key="primary_key", primary_key_type="primary_key_type")
    assert table.type == "type"
    assert table.db_name == "db_name"
    assert table.table_name == "table_name"
    assert table.schema == "schema"
    assert table.count == 12345
    assert table.estimated_storage_size == 67890
    assert table.primary_key == "primary_key"
    assert table.primary_key_type == "primary_key_type"
    assert table.database_name == "db_name"
    assert table.name == "table_name"
    assert table.created_at == "created_at"
    assert table.updated_at == "updated_at"
    assert table.last_import == "last_import"
    assert table.last_log_timestamp == "last_log_timestamp"
    assert table.expire_days == "expire_days"
    assert table.identifier == "db_name.table_name"

def test_schema():
    client = mock.MagicMock()
    job = model.Job(client, "job_id", "type", "query", status="status", url="url", debug="debug", start_at="start_at", end_at="end_at", cpu_time="cpu_time", result_size="result_size", result="result", result_url="result_url", hive_result_schema="hive_result_schema", priority="priority", retry_limit="retry_limit", org_name="org_name", db_name="db_name")
    assert job.job_id == "job_id"
    assert job.type == "type"
    assert job.result_url == "result_url"
    assert job.priority == "priority"
    assert job.retry_limit == "retry_limit"
    assert job.org_name == "org_name"
    assert job.db_name == "db_name"

def test_scheduled_job():
    client = mock.MagicMock()
    schedule = model.Schedule(client, "name", "cron", "query", database="database", result_url="result_url", timezone="timezone", delay="delay", next_time="next_time", priority="priority", retry_limit="retry_limit", org_name="org_name")
    assert schedule.name == "name"
    assert schedule.cron == "cron"
    assert schedule.query == "query"
    assert schedule.database == "database"
    assert schedule.result_url == "result_url"
    assert schedule.timezone == "timezone"
    assert schedule.delay == "delay"
    assert schedule.priority == "priority"
    assert schedule.retry_limit == "retry_limit"
    assert schedule.org_name == "org_name"

def test_result():
    client = mock.MagicMock()
    result = model.Result(client, "name", "url", "org_name")
    assert result.name == "name"
    assert result.url == "url"
    assert result.org_name == "org_name"

def test_bulk_import():
    client = mock.MagicMock()
    bulk_import = model.BulkImport(client, name="name", database="database", table="table", status="status", upload_frozen="upload_frozen", job_id="job_id", valid_records="valid_records", error_records="error_records", valid_parts="valid_parts", error_parts="error_parts")
    assert bulk_import.name == "name"
    assert bulk_import.database == "database"
    assert bulk_import.table == "table"
    assert bulk_import.status == "status"
    assert bulk_import.job_id == "job_id"
    assert bulk_import.valid_records == "valid_records"
    assert bulk_import.error_records == "error_records"
    assert bulk_import.valid_parts == "valid_parts"
    assert bulk_import.error_parts == "error_parts"
    assert bulk_import.upload_frozen == "upload_frozen"

def test_user():
    client = mock.MagicMock()
    user = model.User(client, "name", "org_name", ["role1", "role2"], "email")
    assert user.name == "name"
    assert user.org_name == "org_name"
    assert user.role_names == ["role1", "role2"]
    assert user.email == "email"

def test_access_control():
    client = mock.MagicMock()
    access_control = model.AccessControl(client, "subject", "action", "scope", "grant_option")
    assert access_control.subject == "subject"
    assert access_control.action == "action"
    assert access_control.scope == "scope"
    assert access_control.grant_option == "grant_option"



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    import urllib.parse as urlparse # >=3.0
except ImportError:
    import urlparse
try:
    from urllib.parse import quote as urlquote # >=3.0
except ImportError:
    from urllib import quote as urlquote

class ImportAPI(object):
    ####
    ## Import API
    ##

    def import_data(self, db, table, format, stream, size, unique_id=None):
        """
        TODO: add docstring
        => time:float
        """
        if unique_id is not None:
            path = "/v3/table/import_with_id/%s/%s/%s/%s" % (urlquote(str(db)), urlquote(str(table)), urlquote(str(unique_id)), urlquote(str(format)))
        else:
            path = "/v3/table/import/%s/%s/%s" % (urlquote(str(db)), urlquote(str(table)), urlquote(str(format)))

        kwargs = {}
        if self._endpoint == self.DEFAULT_ENDPOINT:
            kwargs["endpoint"] = self.DEFAULT_IMPORT_ENDPOINT
        with self.put(path, stream, size, **kwargs) as res:
            code, body = res.status, res.read()
            if code / 100 != 2:
                self.raise_error("Import failed", res, body)
            js = self.checked_json(body, ["elapsed_time"])
            time = float(js["elapsed_time"])
            return time



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

class Model(object):
    def __init__(self, client):
        self._client = client

    @property
    def client(self):
        """
        TODO: add docstring
        """
        return self._client

class Account(Model):
    """
    TODO: add docstring
    """

    def __init__(self, client, account_id, plan, storage_size=None, guaranteed_cores=None, maximum_cores=None, created_at=None):
        super(Account, self).__init__(client)
        self._account_id = account_id
        self._plan = plan
        self._storage_size = storage_size
        self._guaranteed_cores = guaranteed_cores
        self._maximum_cores = maximum_cores
        self._created_at = created_at

    @property
    def account_id(self):
        """
        TODO: add docstring
        """
        return self._account_id

    @property
    def plan(self):
        """
        TODO: add docstring
        """
        return self._plan

    @property
    def storage_size(self):
        """
        TODO: add docstring
        """
        return self._storage_size

    @property
    def guaranteed_cores(self):
        """
        TODO: add docstring
        """
        return self._guaranteed_cores

    @property
    def maximum_cores(self):
        """
        TODO: add docstring
        """
        return self._maximum_cores

    @property
    def created_at(self):
        """
        TODO: add docstring
        """
        return self._created_at

    @property
    def storage_size_string(self):
        """
        TODO: add docstring
        """
        if self._storage_size <= 1024 * 1024:
            return "0.0 GB"
        elif self._storage_size <= 60 * 1024 * 1024:
            return "0.01 GB"
        elif self._storage_size <= 60 * 1024 * 1024 * 1024:
            return "%.1f GB" % (float(self._storage_size) / (1024 * 1024 * 1024))
        else:
            return "%d GB" % int(float(self._storage_size) / (1024 * 1024 * 1024))

class Database(Model):
    """
    TODO: add docstring
    """

    PERMISSIONS = ["administrator", "full_access", "import_only", "query_only"]
    PERMISSION_LIST_TABLES = ["administrator", "full_access"]

    def __init__(self, client, db_name, tables=None, count=None, created_at=None, updated_at=None, org_name=None, permission=None):
        super(Database, self).__init__(client)
        self._db_name = db_name
        self._tables = tables
        self._count = count
        self._created_at = created_at
        self._updated_at = updated_at
        self._org_name = org_name
        self._permission = permission

    @property
    def org_name(self):
        """
        TODO: add docstring
        """
        return self._org_name

    @property
    def permission(self):
        """
        TODO: add docstring
        """
        return self._permission

    @property
    def count(self):
        """
        TODO: add docstring
        """
        return self._count

    @property
    def name(self):
        """
        TODO: add docstring
        """
        return self._db_name

    def tables(self):
        """
        TODO: add docstring
        """
        if self._tables is None:
            self._update_tables()
        return self._tables

    def create_log_table(self, name):
        """
        TODO: add docstring
        """
        return self._client.create_log_table(self._db_name, name)

    def create_item_table(self, name):
        """
        TODO: add docstring
        """
        return self._client.create_item_table(self._db_name, name)

    def table(self, table_name):
        """
        TODO: add docstring
        """
        return self._client.table(self._db_name, table_name)

    def delete(self):
        """
        TODO: add docstring
        """
        return self._client.delete_database(self._db_name)

    def query(self, q):
        """
        TODO: add docstring
        """
        return self._client.query(self._db_name, q)

    @property
    def created_at(self):
        """
        TODO: add docstring
        """
        return self._created_at

    @property
    def updated_at(self):
        """
        TODO: add docstring
        """
        return self._updated_at

    def _update_tables(self):
        self._tables = self._client.tables(self._db_name)
        for table in self._tables:
            table.database = self

class Table(Model):
    """
    TODO: add docstring
    """

    def __init__(self, client, db_name, table_name, type, schema, count, created_at=None, updated_at=None, estimated_storage_size=None,
                 last_import=None, last_log_timestamp=None, expire_days=None, primary_key=None, primary_key_type=None):
        super(Table, self).__init__(client)
        self.database = None
        self._db_name = db_name
        self._table_name = table_name
        self._type = type
        self._schema = schema
        self._count = count
        self._created_at = created_at
        self._updated_at = updated_at
        self._estimated_storage_size = estimated_storage_size
        self._last_import = last_import
        self._last_log_timestamp = last_log_timestamp
        self._expire_days = expire_days
        self._primary_key = primary_key
        self._primary_key_type = primary_key_type

    @property
    def type(self):
        """
        TODO: add docstring
        """
        return self._type

    @property
    def db_name(self):
        """
        TODO: add docstring
        """
        return self._db_name

    @property
    def table_name(self):
        """
        TODO: add docstring
        """
        return self._table_name

    @property
    def schema(self):
        """
        TODO: add docstring
        """
        return self._schema

    @property
    def count(self):
        """
        TODO: add docstring
        """
        return self._count

    @property
    def estimated_storage_size(self):
        """
        TODO: add docstring
        """
        return self._estimated_storage_size

    @property
    def primary_key(self):
        """
        TODO: add docstring
        """
        return self._primary_key

    @property
    def primary_key_type(self):
        """
        TODO: add docstring
        """
        return self._primary_key_type

    @property
    def database_name(self):
        """
        TODO: add docstring
        """
        return self._db_name

    @property
    def name(self):
        """
        TODO: add docstring
        """
        return self._table_name

    @property
    def created_at(self):
        """
        TODO: add docstring
        """
        return self._created_at

    @property
    def updated_at(self):
        """
        TODO: add docstring
        """
        return self._updated_at

    @property
    def last_import(self):
        """
        TODO: add docstring
        """
        return self._last_import

    @property
    def last_log_timestamp(self):
        """
        TODO: add docstring
        """
        return self._last_log_timestamp

    @property
    def expire_days(self):
        """
        TODO: add docstring
        """
        return self._expire_days

    @property
    def permission(self):
        """
        TODO: add docstring
        """
        if self.database is None:
            self._update_database()
        return self.database.permission

    @property
    def identifier(self):
        """
        TODO: add docstring
        """
        return "%s.%s" % (self._db_name, self._table_name)

    def delete(self):
        """
        TODO: add docstring
        """
        return self._client.delete_table(self._db_name, self._table_name)

    def tail(self, count, to=None, _from=None):
        """
        TODO: add docstring
        """
        return self._client.tail(self._db_name, self._table_name, count, to, _from)

    def import_data(self, format, stream, size):
        """
        TODO: add docstring
        """
        return self._client.import_data(self._db_name, self._table_name, format, stream, size)

    def export_data(self, storage_type, **kwargs):
        """
        TODO: add docstring
        """
        return self._client.export_data(self._db_name, self._table_name, storage_type, kwargs)

    @property
    def estimated_storage_size_string(self):
        """
        TODO: add docstring
        """
        if self._estimated_storage_size <= 1024*1024:
            return "0.0 GB"
        elif self._estimated_storage_size <= 60*1024*1024:
            return "0.01 GB"
        elif self._estimated_storage_size <= 60*1024*1024*1024:
            return "%.1f GB" % (float(self._estimated_storage_size) / (1024*1024*1024))
        else:
            return "%d GB" % int(float(self._estimated_storage_size) / (1024*1024*1024))

    def _update_database(self):
        self.database = self._client.database(self._db_name)

class Schema(object):
    """
    TODO: add docstring
    """

    class Field(object):
        def __init__(self, name, type):
            self._name = name
            self._type = type

        @property
        def name(self):
            """
            TODO: add docstring
            """
            return self._name

        @property
        def type(self):
            """
            TODO: add docstring
            """
            return self._type

    def __init__(self, fields=[]):
        self._fields = fields

    @property
    def fields(self):
        """
        TODO: add docstring
        """
        return self._fields

    def add_field(self, name, type):
        """
        TODO: add docstring
        """
        self._fields.append(Field(name, type))

class Job(Model):
    """
    TODO: add docstring
    """

    STATUS_QUEUED = "queued"
    STATUS_BOOTING = "booting"
    STATUS_RUNNING = "running"
    STATUS_SUCCESS = "success"
    STATUS_ERROR = "error"
    STATUS_KILLED = "killed"
    FINISHED_STATUS = [STATUS_SUCCESS, STATUS_ERROR, STATUS_KILLED]

    def __init__(self, client, job_id, type, query, status=None, url=None, debug=None, start_at=None, end_at=None, cpu_time=None,
                 result_size=None, result=None, result_url=None, hive_result_schema=None, priority=None, retry_limit=None,
                 org_name=None, db_name=None):
        super(Job, self).__init__(client)
        self._job_id = job_id
        self._type = type
        self._url = url
        self._query = query
        self._status = status
        self._debug = debug
        self._start_at = start_at
        self._end_at = end_at
        self._cpu_time = cpu_time
        self._result_size = result_size
        self._result = result
        self._result_url = result_url
        self._hive_result_schema = hive_result_schema
        self._priority = priority
        self._retry_limit = retry_limit
        self._org_name = org_name
        self._db_name = db_name

    @property
    def job_id(self):
        """
        TODO: add docstring
        """
        return self._job_id

    @property
    def type(self):
        """
        TODO: add docstring
        """
        return self._type

    @property
    def result_url(self):
        """
        TODO: add docstring
        """
        return self._result_url

    @property
    def priority(self):
        """
        TODO: add docstring
        """
        return self._priority

    @property
    def retry_limit(self):
        """
        TODO: add docstring
        """
        return self._retry_limit

    @property
    def org_name(self):
        """
        TODO: add docstring
        """
        return self._org_name

    @property
    def db_name(self):
        """
        TODO: add docstring
        """
        return self._db_name

    def wait(self, timeout=None):
        """
        TODO: add docstring
        """
        raise NotImplementedError # TODO

    def kill(self):
        """
        TODO: add docstring
        """
        raise NotImplementedError # TODO

    def query(self):
        """
        TODO: add docstring
        """
        if self._query is not None and not self.finished():
            self._update_status()
        return self._query

    def status(self):
        """
        TODO: add docstring
        """
        if self._query is not None and not self.finished():
            self._update_status()
        return self._status

    @property
    def url(self):
        """
        TODO: add docstring
        """
        return self._url

    def result(self):
        """
        TODO: add docstring
        """
        if not self.finished():
            pass
        else:
            if self._result is None:
                for row in self._client.job_result_each(self._job_id):
                    yield row
            else:
                for row in self._result:
                    yield row

    def result_format(self, format):
        """
        TODO: add docstring
        """
        if not self.finished():
            pass
        else:
            if self._result is None:
                for row in self._client.job_result_format_each(self._job_id, format):
                    yield row
            else:
                for row in self._result:
                    yield row

    def finished(self):
        """
        TODO: add docstring
        """
        if self._status not in self.FINISHED_STATUS:
            self._update_progress()
        return self._status in self.FINISHED_STATUS

    def success(self):
        """
        TODO: add docstring
        """
        if self._status not in self.FINISHED_STATUS:
            self._update_progress()
        return self._status == self.STATUS_SUCCESS

    def error(self):
        """
        TODO: add docstring
        """
        if self._status not in self.FINISHED_STATUS:
            self._update_progress()
        return self._status == self.STATUS_ERROR

    def killed(self):
        """
        TODO: add docstring
        """
        if self._status not in self.FINISHED_STATUS:
            self._update_progress()
        return self._status == self.STATUS_KILLED

    def queued(self):
        """
        TODO: add docstring
        """
        if self._status not in self.FINISHED_STATUS:
            self._update_progress()
        return self._status == self.STATUS_QUEUED

    def running(self):
        """
        TODO: add docstring
        """
        if self._status not in self.FINISHED_STATUS:
            self._update_progress()
        return self._status == self.STATUS_RUNNING

    def _update_progress(self):
        self._status = self._client.job_status(self._job_id)

    def _update_status(self):
        type, query, status, url, debug, start_at, end_at, cpu_time, result_size, result_url, hive_result_schema, priority, retry_limit, org_name, db_name = self._client.api.show_job(self._job_id)
        self._query = query
        self._status = status
        self._url = url
        self._debug = debug
        self._start_at = start_at
        self._end_at = end_at
        self._cpu_time = cpu_time
        self._result_size = result_size
        self._result_url = result_url
        self._hive_result_schema = hive_result_schema
        self._priority = priority
        self._retry_limit = retry_limit
        self._db_name = db_name

class ScheduledJob(Job):
    def __init__(self, client, scheduled_at, *args, **kwargs):
        super(ScheduledJob, self).__init__(client, *args, **kwargs)
        self._scheduled_at = scheduled_at

    @property
    def scheduled_at(self):
        """
        TODO: add docstring
        """
        return self._created_at

class Schedule(Model):
    """
    TODO: add docstring
    """

    def __init__(self, client, name, cron, query, database=None, result_url=None, timezone=None, delay=None, next_time=None, priority=None, retry_limit=None, org_name=None):
        super(Schedule, self).__init__(client)
        self._name = name
        self._cron = cron
        self._query = query
        self._database = database
        self._result_url = result_url
        self._timezone = timezone
        self._delay = delay
        self._next_time = next_time
        self._priority = priority
        self._retry_limit = retry_limit
        self._org_name = org_name

    @property
    def name(self):
        """
        TODO: add docstring
        """
        return self._name

    @property
    def cron(self):
        """
        TODO: add docstring
        """
        return self._cron

    @property
    def query(self):
        """
        TODO: add docstring
        """
        return self._query

    @property
    def database(self):
        """
        TODO: add docstring
        """
        return self._database

    @property
    def result_url(self):
        """
        TODO: add docstring
        """
        return self._result_url

    @property
    def timezone(self):
        """
        TODO: add docstring
        """
        return self._timezone

    @property
    def delay(self):
        """
        TODO: add docstring
        """
        return self._delay

    @property
    def priority(self):
        """
        TODO: add docstring
        """
        return self._priority

    @property
    def retry_limit(self):
        """
        TODO: add docstring
        """
        return self._retry_limit

    @property
    def org_name(self):
        """
        TODO: add docstring
        """
        return self._org_name

    @property
    def next_time(self):
        """
        TODO: add docstring
        """
        return self._next_time

    def run(self, time, num):
        """
        TODO: add docstring
        """
        return self._client.run_schedule(time, num)

class Result(Model):
    """
    TODO: add docstring
    """

    def __init__(self, client, name, url, org_name):
        super(Result, self).__init__(client)
        self._name = name
        self._url = url
        self._org_name = org_name

    @property
    def name(self):
        """
        TODO: add docstring
        """
        return self._name

    @property
    def url(self):
        """
        TODO: add docstring
        """
        return self._url

    @property
    def org_name(self):
        """
        TODO: add docstring
        """
        return self._org_name

class BulkImport(Model):
    """
    TODO: add docstring
    """

    def __init__(self, client, name=None, database=None, table=None, status=None, upload_frozen=None, job_id=None, valid_records=None, error_records=None, valid_parts=None, error_parts=None, **kwargs):
        super(BulkImport, self).__init__(client)
        self._name = name
        self._database = database
        self._table = table
        self._status = status
        self._upload_frozen = upload_frozen
        self._job_id = job_id
        self._valid_records = valid_records
        self._error_records = error_records
        self._valid_parts = valid_parts
        self._error_parts = error_parts

    @property
    def name(self):
        """
        TODO: add docstring
        """
        return self._name

    @property
    def database(self):
        """
        TODO: add docstring
        """
        return self._database

    @property
    def table(self):
        """
        TODO: add docstring
        """
        return self._table

    @property
    def status(self):
        """
        TODO: add docstring
        """
        return self._status

    @property
    def job_id(self):
        """
        TODO: add docstring
        """
        return self._job_id

    @property
    def valid_records(self):
        """
        TODO: add docstring
        """
        return self._valid_records

    @property
    def error_records(self):
        """
        TODO: add docstring
        """
        return self._error_records

    @property
    def valid_parts(self):
        """
        TODO: add docstring
        """
        return self._valid_parts

    @property
    def error_parts(self):
        """
        TODO: add docstring
        """
        return self._error_parts

    @property
    def upload_frozen(self):
        """
        TODO: add docstring
        """
        return self._upload_frozen

    def error_record_items(self):
        """
        TODO: add docstring
        """
        for record in self._client.bulk_import_error_records(self.name):
            yield record

class User(Model):
    """
    TODO: add docstring
    """

    def __init__(self, client, name, org_name, role_names, email):
        super(User, self).__init__(client)
        self._name = name
        self._org_name = org_name
        self._role_names = role_names
        self._email = email

    @property
    def name(self):
        """
        TODO: add docstring
        """
        return self._name

    @property
    def org_name(self):
        """
        TODO: add docstring
        """
        return self._org_name

    @property
    def role_names(self):
        """
        TODO: add docstring
        """
        return self._role_names

    @property
    def email(self):
        """
        TODO: add docstring
        """
        return self._email

class AccessControl(Model):
    """
    TODO: add docstring
    """

    def __init__(self, client, subject, action, scope, grant_option):
        super(AccessControl, self).__init__(client)
        self._subject = subject
        self._action = action
        self._scope = scope
        self._grant_option = grant_option

    @property
    def subject(self):
        """
        TODO: add docstring
        """
        return self._subject

    @property
    def action(self):
        """
        TODO: add docstring
        """
        return self._action

    @property
    def scope(self):
        """
        TODO: add docstring
        """
        return self._scope

    @property
    def grant_option(self):
        """
        TODO: add docstring
        """
        return self._grant_option



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    from unittest import mock
except ImportError:
    import mock
import pytest

from tdclient import api
from tdclient.test.test_helper import *

def setup_function(function):
    unset_environ()

def test_authenticate_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "apikey": "foobarbaz"
        }
    """
    td.post = mock.MagicMock(return_value=make_response(200, body))
    apikey = td.authenticate("foo", "bar")
    td.post.assert_called_with("/v3/user/authenticate", {"user": "foo", "password": "bar"})
    assert apikey == "foobarbaz"

def test_list_users_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "users":[
                {"name":"foo","email":"foo@example.com"},
                {"name":"bar","email":"foo@example.com"},
                {"name":"baz","email":"foo@example.com"}
            ]
        }
    """
    td.get = mock.MagicMock(return_value=make_response(200, body))
    users = td.list_users()
    td.get.assert_called_with("/v3/user/list")
    assert len(users) == 3

def test_list_users_failure():
    td = api.API("APIKEY")
    td.get = mock.MagicMock(return_value=make_response(500, b"error"))
    with pytest.raises(api.APIError) as error:
        td.list_users()
    assert error.value.args == ("500: List users failed: error",)

def test_add_user_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    apikey = td.add_user("name", "org", "email@example.com", "password")
    td.post.assert_called_with("/v3/user/add/name", {"organization": "org", "email": "email@example.com", "password": "password"})

def test_remove_user_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    apikey = td.remove_user("user")
    td.post.assert_called_with("/v3/user/remove/user")

def test_change_email_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    apikey = td.change_email("user", "email@example.com")
    td.post.assert_called_with("/v3/user/email/change/user", {"email": "email@example.com"})

def test_list_apikeys_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "apikeys": []
        }
    """
    td.get = mock.MagicMock(return_value=make_response(200, body))
    apikeys = td.list_apikeys("foo")
    td.get.assert_called_with("/v3/user/apikey/list/foo")

def test_add_apikey_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    apikeys = td.add_apikey("foo")
    td.post.assert_called_with("/v3/user/apikey/add/foo")

def test_remove_apikey_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    apikeys = td.remove_apikey("foo", "bar")
    td.post.assert_called_with("/v3/user/apikey/remove/foo", {"apikey": "bar"})

def test_change_password_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    apikeys = td.change_password("foo", "bar")
    td.post.assert_called_with("/v3/user/password/change/foo", {"password": "bar"})

def test_change_my_password_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    apikeys = td.change_my_password("foo", "bar")
    td.post.assert_called_with("/v3/user/password/change", {"old_password": "foo", "password": "bar"})



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    from unittest import mock
except ImportError:
    import mock
import pytest

from tdclient import api
from tdclient.test.test_helper import *

def setup_function(function):
    unset_environ()

def test_partial_delete_success():
    td = api.API("APIKEY")
    # TODO: should be replaced by wire dump
    body = b"""
        {
            "job_id":"12345"
        }
    """
    td.post = mock.MagicMock(return_value=make_response(200, body))
    partial_delete = td.partial_delete("sample_datasets", "nasdaq", 0, 10)
    td.post.assert_called_with("/v3/table/partialdelete/sample_datasets/nasdaq", {"from": "10", "to": "0"})

def test_partial_delete_failure():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(500, b"error"))
    with pytest.raises(api.APIError) as error:
        td.partial_delete("sample_datasets", "nasdaq", 0, 10)
    assert error.value.args == ("500: Partial delete failed: error",)



#!/usr/bin/env python

from __future__ import with_statement
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
import re
import sys

version = None

with open("tdclient/version.py") as fp:
    m = re.search(r"^__version__ *= *\"([^\"]*)\" *$", fp.read(), re.MULTILINE)
    if m is not None:
        version = m.group(1)

if version is None:
    raise(RuntimeError("could not read tdclient/version.py"))

class PyTest(TestCommand):
    user_options = [("pytest-args=", "a", "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)

install_requires = []
with open("requirements.txt") as fp:
    for s in fp:
        install_requires.append(s.strip())

test_require = []
with open("test-requirements.txt") as fp:
    for s in fp:
        test_require.append(s.strip())

setup(
    name="td-client",
    version=version,
    description="Treasure Data API library for Python",
    author="Treasure Data, Inc.",
    author_email="support@treasure-data.com",
    url="http://treasuredata.com/",
    install_requires=install_requires,
    tests_require=test_require,
    packages=find_packages(),
    cmdclass = {"test": PyTest},
    license="Apache Software License",
    platforms="Posix; MacOS X; Windows",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Topic :: Internet",
    ],
)



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    from urllib.parse import quote as urlquote # >=3.0
except ImportError:
    from urllib import quote as urlquote

class DatabaseAPI(object):
    ####
    ## Database API
    ##

    def list_databases(self):
        """
        TODO: add docstring
        => [name:str]
        """
        with self.get("/v3/database/list") as res:
            code, body = res.status, res.read()
            if code != 200:
                self.raise_error("List databases failed", res, body)
            js = self.checked_json(body, ["databases"])
            result = {}
            for m in js["databases"]:
                name = m.get("name")
                count = m.get("count")
                created_at = self.parsedate(self.get_or_else(m, "created_at", "1970-01-01T00:00:00Z"))
                updated_at = self.parsedate(self.get_or_else(m, "updated_at", "1970-01-01T00:00:00Z"))
                permission = m.get("permission")
                result[name] = [count, created_at, updated_at, None, permission] # set None to org for API copatibility
            return result

    def delete_database(self, db):
        """
        TODO: add docstring
        => True
        """
        with self.post("/v3/database/delete/%s" % urlquote(str(db))) as res:
            code, body = res.status, res.read()
            if code != 200:
                self.raise_error("Delete database failed", res, body)
            return True

    def create_database(self, db, params={}):
        """
        TODO: add docstring
        => True
        """
        with self.post("/v3/database/create/%s" % urlquote(str(db)), params) as res:
            code, body = res.status, res.read()
            if code != 200:
                self.raise_error("Create database failed", res, body)
            return True



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    from unittest import mock
except ImportError:
    import mock
import pytest

from tdclient import api
from tdclient.test.test_helper import *

def setup_function(function):
    unset_environ()

def test_list_databases_success():
    td = api.API("APIKEY")
    body = b"""
        {
            "databases":[
                {"name":"sample_datasets","count":8812278,"created_at":"2014-10-04 01:13:11 UTC","updated_at":"2014-10-08 18:42:12 UTC","organization":null,"permission":"administrator"},
                {"name":"jma_weather","count":29545,"created_at":"2014-10-12 06:33:01 UTC","updated_at":"2014-10-12 06:33:01 UTC","organization":null,"permission":"administrator"},
                {"name":"test_db","count":15,"created_at":"2014-10-16 09:48:44 UTC","updated_at":"2014-10-16 09:48:44 UTC","organization":null,"permission":"administrator"}
            ]
        }
    """
    td.get = mock.MagicMock(return_value=make_response(200, body))
    databases = td.list_databases()
    td.get.assert_called_with("/v3/database/list")
    assert len(databases) == 3
    assert sorted(databases.keys()) == ["jma_weather", "sample_datasets", "test_db"]
    assert sorted([ v[0] for v in databases.values() ]) == [15, 29545, 8812278]

def test_list_databases_failure():
    td = api.API("APIKEY")
    td.get = mock.MagicMock(return_value=make_response(500, b"error"))
    with pytest.raises(api.APIError) as error:
        td.list_databases()
    assert error.value.args == ("500: List databases failed: error",)

def test_delete_database_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    td.delete_database("sample_datasets")
    td.post.assert_called_with("/v3/database/delete/sample_datasets")

def test_create_database_success():
    td = api.API("APIKEY")
    td.post = mock.MagicMock(return_value=make_response(200, b""))
    td.create_database("sample_datasets")
    td.post.assert_called_with("/v3/database/create/sample_datasets", {})



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

import codecs
import json
import msgpack
try:
    from urllib.parse import quote as urlquote # >=3.0
except ImportError:
    from urllib import quote as urlquote

class JobAPI(object):
    ####
    ## Job API
    ##

    def list_jobs(self, _from=0, to=None, status=None, conditions=None):
        """
        TODO: add docstring
        => [(jobId:str, type:str, status:str, start_at:str, end_at:str, result_url:str)]
        """
        params = {}
        if _from is not None:
            params["from"] = str(_from)
        if to is not None:
            params["to"] = str(to)
        if status is not None:
            params["status"] = str(status)
        if conditions is not None:
            params.update(conditions)
        with self.get("/v3/job/list", params) as res:
            code, body = res.status, res.read()
            if code != 200:
                self.raise_error("List jobs failed", res, body)
            js = self.checked_json(body, ["jobs"])
            result = []
            for m in js["jobs"]:
                job_id = m.get("job_id")
                type = m.get("type", "?")
                database = m.get("database")
                status = m.get("status")
                query = m.get("query")
                start_at = self.parsedate(self.get_or_else(m, "start_at", "1970-01-01T00:00:00Z"))
                end_at = self.parsedate(self.get_or_else(m, "end_at", "1970-01-01T00:00:00Z"))
                cpu_time = m.get("cpu_time")
                result_size = m.get("result_size") # compressed result size in msgpack.gz format
                result_url = m.get("result")
                priority = m.get("priority")
                retry_limit = m.get("retry_limit")
                result.append((job_id, type, status, query, start_at, end_at, cpu_time,
                     result_size, result_url, priority, retry_limit, None, database))
            return result

    def show_job(self, job_id):
        """
        TODO: add docstring
        => (type:str, status:str, result:str, url:str, result:str)
        """
        # use v3/job/status instead of v3/job/show to poll finish of a job
        with self.get("/v3/job/show/%s" % (urlquote(str(job_id)))) as res:
            code, body = res.status, res.read()
            if code != 200:
                self.raise_error("Show job failed", res, body)
            js = self.checked_json(body, ["status"])
            type = js.get("type", "?")
            database = js.get("database")
            query = js.get("query")
            status = js.get("status")
            debug = js.get("debug")
            url = js.get("url")
            start_at = self.parsedate(self.get_or_else(js, "start_at", "1970-01-01T00:00:00Z"))
            end_at = self.parsedate(self.get_or_else(js, "end_at", "1970-01-01T00:00:00Z"))
            cpu_time = js.get("cpu_time")
            result_size = js.get("result_size") # compressed result size in msgpack.gz format
            result = js.get("result") # result target URL
            hive_result_schema = js.get("hive_result_schema", "")
            if hive_result_schema is None or len(str(hive_result_schema)) < 1:
                hive_result_schema = None
            else:
                hive_result_schema = json.loads(hive_result_schema)
            priority = js.get("priority")
            retry_limit = js.get("retry_limit")
            return (type, query, status, url, debug, start_at, end_at, cpu_time,
                    result_size, result, hive_result_schema, priority, retry_limit, None, database)

    def job_status(self, job_id):
        """
        TODO: add docstring
        """
        with self.get("/v3/job/status/%s" % (urlquote(str(job_id)))) as res:
            code, body = res.status, res.read()
            if code != 200:
                self.raise_error("Get job status failed", res, body)

            js = self.checked_json(body, ["status"])
            return js["status"]

    def job_result(self, job_id):
        """
        TODO: add docstring
        """
        result = []
        for row in self.job_result_format_each(job_id, "msgpack"):
            result.append(row)
        return result

    def job_result_each(self, job_id):
        """
        TODO: add docstring
        """
        for row in self.job_result_format_each(job_id, "msgpack"):
            yield row

    def job_result_format(self, job_id, format):
        """
        TODO: add docstring
        """
        result = []
        for row in self.job_result_format_each(job_id, format):
            result.append(row)
        return result

    def job_result_format_each(self, job_id, format):
        """
        TODO: add docstring
        """
        with self.get("/v3/job/result/%s" % (urlquote(str(job_id))), {"format": format}) as res:
            code = res.status
            if code != 200:
                self.raise_error("Get job result failed", res)
            if format == "msgpack":
                unpacker = msgpack.Unpacker(res, encoding=str("utf-8"))
                for row in unpacker:
                    yield row
            elif format == "json":
                unpacker = json.load(codecs.getreader("utf-8")(res))
                for row in unpacker:
                    yield row
            else:
                yield res.read()

    def kill(self, job_id):
        """
        TODO: add docstring
        """
        with self.post("/v3/job/kill/%s" % (urlquote(str(job_id)))) as res:
            code, body = res.status, res.read()
            if code != 200:
                self.raise_error("Kill job failed", res, body)
            js = self.checked_json(body, [])
            former_status = js.get("former_status")
            return former_status

    def hive_query(self, q, db=None, result_url=None, priority=None, retry_limit=None, **kwargs):
        """
        TODO: add docstring
        => jobId:str
        """
        return self.query(q, "hive", db, result_url, priority, retry_limit, **kwargs)

    def pig_query(self, q, db=None, result_url=None, priority=None, retry_limit=None, **kwargs):
        """
        TODO: add docstring
        => jobId:str
        """
        return self.query(q, "pig", db, result_url, priority, retry_limit, **kwargs)

    def query(self, q, type="hive", db=None, result_url=None, priority=None, retry_limit=None, **kwargs):
        """
        TODO: add docstring
        => jobId:str
        """
        params = {"query": q}
        params.update(kwargs)
        if result_url is not None:
            params["result"] = result_url
        if priority is not None:
            params["priority"] = priority
        if retry_limit is not None:
            params["retry_limit"] = retry_limit
        with self.post("/v3/job/issue/%s/%s" % (urlquote(str(type)), urlquote(str(db))), params) as res:
            code, body = res.status, res.read()
            if code != 200:
                self.raise_error("Query failed", res, body)
            js = self.checked_json(body, ["job_id"])
            return str(js["job_id"])



#!/usr/bin/env python

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import with_statement

try:
    from urllib.parse import quote as urlquote # >=3.0
except ImportError:
    from urllib import quote as urlquote

class ResultAPI(object):
    ####
    ## Result API
    ##

    def list_result(self):
        """
        TODO: add docstring
        """
        with self.get("/v3/result/list") as res:
            code, body = res.status, res.read()
            if code != 200:
                self.raise_error("List result table failed", res, body)
            js = self.checked_json(body, ["results"])
            return [ (m["name"], m["url"], None) for m in js["result"] ] # same as database

    def create_result(self, name, url, params={}):
        """
        TODO: add docstring
        => True
        """
        params.update({"url": url})
        with self.post("/v3/result/create/%s" % (urlquote(str(name))), params) as res:
            code, body = res.status, res.read()
            if code != 200:
                self.raise_error("Create result table failed", res, body)
            return True

    def delete_result(self, name):
        """
        TODO: add docstring
        => True
        """
        with self.post("/v3/result/delete/%s" % (urlquote(str(name)))) as res:
            code, body = res.status, res.read()
            if code != 200:
                self.raise_error("Delete result table failed", res, body)
            return True
